<'
# Copyright (c) 2012-2014 LAAS/CNRS
# All rights reserved.
#
# Redistribution and use  in source  and binary  forms,  with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   1. Redistributions of  source  code must retain the  above copyright
#      notice and this list of conditions.
#   2. Redistributions in binary form must reproduce the above copyright
#      notice and  this list of  conditions in the  documentation and/or
#      other materials provided with the distribution.
#
#                                      Anthony Mallet on Mon Apr  9 2012
#

if {[llength $argv] != 1} { error "expected arguments: component" }
lassign $argv component

# compute handy shortcuts
set comp [$component name]

lang c
'>
#include "autoconf/acheader.h"

#include <assert.h>
#include <errno.h>
#include <inttypes.h>
#include <limits.h>
#include <math.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "h2errorLib.h"

#include "<"$comp">/c/client.h"
#include "<"$comp">_json-types.h"
#include "private.h"

<'foreach s [$component services] {'>

/* <"[--- Service [$s name] encoding/decoding ---------------------------]"> */

int
genom_<"$comp">_client_<"[$s name]">_json_scan(
  struct genom_<"$comp">_<"[$s name]">_input *in, const char *json,
  char **endptr)
{
  int s = 0;
  json_skip_whitespace(json);
  if (*(json++) != '{') { s = EINVAL; goto done; }
  json_skip_whitespace(json);
  while(*json != '}') {
    if (*(json++) != '"') { s = EINVAL; goto done; };
<'    foreach p [$s parameters in inout] {'>
    if (!strncmp(json, "<"[$p name]">\"", 1+<"[string length [$p name]]">)) {
      json += 1+<"[string length [$p name]]">;
      json_skip_whitespace(json);
      if (*(json++) != ':') { s = EINVAL; goto done; }
      if ((s = json_scan_<"[[$p type] mangle]">(
             <"[[$p type] pass reference in->[$p name]]">, &json)))
        goto done;
    } else
<'    }'>
    { s = EINVAL; goto done; }

    json_skip_whitespace(json);
    if (*json == ',') {
      json++;
      json_skip_whitespace(json);
    }
  }

done:
  if (endptr) *endptr = (char *)json;
  return s;
}

int
genom_<"$comp">_client_<"[$s name]">_json_print(char **json,
             const struct genom_<"$comp">_<"[$s name]">_output *out)
{
  size_t len;
  char *end;
  int s;

  len = JSON_MINBUF;
  *json = end = malloc(len);
  if (!*json) return ENOMEM;

  if ((s = bufcat(json, &end, &len, 0, "{"))) goto done;

<'  set sep {}'>
<'  foreach p [$s parameters inout out] {'>
  if ((s = bufcat(json, &end, &len, 0, "<"$sep">\"<"[$p name]">\":")))
    goto done;
  if ((s = json_print_<"[[$p type] mangle]">(json, &end, &len,
         <"[[$p type] pass value out->[$p name]]">)))
    goto done;
<'  set sep ,'>
<'    }'>

  if ((s = bufcat(json, &end, &len, 0, "}"))) goto done;

done:
  if (s && *json) {
    free(*json);
    *json = NULL;
  }
  return s;
}

<'}'>

<'foreach p [$component port out] {'>

/* <"[--- Port [$p name] encoding ---------------------------------------]"> */

int
genom_<"$comp">_client_<"[$p name]">_json_print(char **json,
  <"[[$p datatype] argument value data]">)
{
  size_t len;
  char *end;
  int s;

  len = JSON_MINBUF;
  *json = end = malloc(len);
  if (!*json) return ENOMEM;

  if ((s = bufcat(json, &end, &len, 0, "{"))) goto done;

  if ((s = bufcat(json, &end, &len, 0, "\"<"[$p name]">\":")))
    goto done;
  if ((s = json_print_<"[[$p datatype] mangle]">(json, &end, &len, data)))
    goto done;

  if ((s = bufcat(json, &end, &len, 0, "}"))) goto done;

done:
  if (s && *json) {
    free(*json);
    *json = NULL;
  }
  return s;
}

<'}'>

/* <"[--- ${comp}_json_error --------------------------------------------]"> */

char *
genom_<"$comp">_client_json_error(genom_client h,
                         genom_event e, const void *detail)
{
  char *s, *end;
  size_t len;

  len = JSON_MINBUF;
  s = end = malloc(len);
  if (!*s) return NULL;

  if (!e && h) detail = h->context.raised(&e, &h->context);

  if (e == genom_syserr_id) {
    if (bufcat(&s, &end, &len, 0, "\"")) return NULL;
    if (bufcat(&s, &end, &len, 1,
               strerror(((genom_syserr_detail *)detail)->code))) return NULL;
    if (bufcat(&s, &end, &len, 0, "\"")) return NULL;
    return s;
  } else if (e == genom_unkex_id) {
    if (bufcat(&s, &end, &len, 0, "{\"ex\":\"")) return NULL;
    if (bufcat(&s, &end, &len, 0, genom_unkex_id)) return NULL;
    if (detail) {
      if (bufcat(&s, &end, &len, 0, "\",\"detail\":{\"what\":\"")) return NULL;
      if (bufcat(&s, &end, &len, 1,
                 ((genom_unkex_detail *)detail)->what)) return NULL;
    }
    if (bufcat(&s, &end, &len, 0, "\"}}")) return NULL;
    return s;
  }
<'foreach e [$component types public] {'>
<'  if {[$e kind] != "exception"} continue'>
  else if (e == <"[$e cname]">_id) {
<'  if {[llength [$e members]]} {'>
    if (bufcat(&s, &end, &len, 0, "{\"ex\":\"")) return NULL;
    if (bufcat(&s, &end, &len, 0, <"[$e cname]">_id)) return NULL;
    if (detail) {
      if (bufcat(&s, &end, &len, 0, "\",\"detail\":")) return NULL;
      if (json_print_<"[$e mangle]">(&s, &end, &len, detail)) return NULL;
    }
    if (bufcat(&s, &end, &len, 0, "}")) return NULL;
<'  } else {'>
    if (bufcat(&s, &end, &len, 0, "{\"ex\":\"")) return NULL;
    if (bufcat(&s, &end, &len, 0, <"[$e cname]">_id)) return NULL;
    if (bufcat(&s, &end, &len, 0, "\"}")) return NULL;
<'  }'>
    return s;
  }
<'}'>

  assert(!"unknown genom exception");
  return NULL;
}
