<'
# Copyright (c) 2011-2014 LAAS/CNRS
# All rights reserved.
#
# Redistribution and use  in source  and binary  forms,  with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   1. Redistributions of  source  code must retain the  above copyright
#      notice and this list of conditions.
#   2. Redistributions in binary form must reproduce the above copyright
#      notice and  this list of  conditions in the  documentation and/or
#      other materials provided with the distribution.
#
#                                      Anthony Mallet on Wed Feb 29 2012
#

if {[llength $argv] != 1} { error "expected arguments: component" }
lassign $argv component

# compute handy shortcuts
set comp [$component name]
set COMP [string toupper [$component name]]

lang c
'>
#ifndef H_<"$COMP">_CONTROL_TASK
#define H_<"$COMP">_CONTROL_TASK

#include <assert.h>
#include <stdarg.h>

#include "csLib.h"
#include "semLib.h"

#include "<"$comp">_c_types.h"
#include "<"$comp">_parameters.h"
#include "<"$comp">_msglib.h"
#include "<"$comp">_portlib.h"
#include "<"$comp">_remotelib.h"
#include "<"$comp">_activity.h"


/* --- internal state ------------------------------------------------------ */

extern const char *genom_instance;

struct genom_component_data;

struct genom_context_data {
  struct genom_component_data *self;

  genom_event ex;
  void *exdetail;
  size_t exsize;
};

struct genom_component_data {
  struct genom_context_iface context;
  struct genom_context_data context_data;
  int shutdown;

  char mbox_name[64];
  SERV_ID csserv;
  long taskid;
  SEM_ID sync;

<'if {![catch {$component ids} ids]} {'>
  <"[$ids declarator ids]">;
<'}'>

  struct {
<'foreach p [$component ports] {'>
    struct genom_<"$comp">_<"[$p name]">_port <"[$p name]">;
<'}'>
  } ports;

  struct {
<'foreach r [$component remotes] {'>
    struct genom_<"$comp">_<"[$r name]">_remote <"[$r name]">;
<'}'>
  } remotes;

  struct {
<'foreach t [$component tasks] {'>
    struct {
      struct genom_context_iface context;
      struct genom_context_data context_data;

      long taskid;
      int stateid;
      int runnable;
      SEM_ID sync;
      genom_event status;
      genom_event pact;
    } <"[$t name]">;
<'}'>
  } tasks;

  /* activities */
  struct genom_activities activities;
  int run_map[<"$COMP">_NRQSTID];
};


/* control task */
void *	genom_<"$comp">_init(void);
void	genom_<"$comp">_fini(void *data);
void	genom_<"$comp">_schedule_cntrl(struct genom_component_data *self);
void	genom_<"$comp">_schedule_exec(struct genom_component_data *self);

<'foreach s [$component services] {'>
genom_event genom_<"$comp">_<"[$s name]">_controlcb(
   struct genom_component_data *,
   struct genom_<"$comp">_<"[$s name]">_activity *);
<'}'>


/* exec tasks */
<'foreach t [$component tasks] {'>
void *		genom_<"$comp">_<"[$t name]">_exec_task(void *data);
genom_event	genom_<"$comp">_<"[$t name]">_invoke_perm(
			struct genom_component_data *);
genom_event	genom_<"$comp">_<"[$t name]">_invoke(
			struct genom_component_data *,
			struct genom_activity *);
<'}'>


/* exceptions */
genom_event	genom_pocolibs_raise(genom_event ex, void *detail, size_t size,
                        genom_context self);
const void *	genom_pocolibs_raised(genom_event *ex, genom_context self);


/* log functions */
void	genom_log_info(const char *format, ...)
  __attribute__ ((format (printf, 1, 2)));
void	genom_log_warn(int h2error, const char *format, ...)
  __attribute__ ((format (printf, 2, 3)));
void	genom_log_debug(const char *format, ...)
  __attribute__ ((format (printf, 1, 2)));


#endif /* H_<"$COMP">_CONTROL_TASK */
